**********************************************************************
*
* Procedure file: RANDDAT.PRG
*
*         System: Random data generator
*
*         Author: C. Brian Near
*                 317 Maple Ave.
*                 Deseronto, ON
*                 CANADA  K0K 1X0
*
*         E-mail: bnear@blvl.igs.net
*                      or
*                 casrsdfd@ibmmail.com
*
*  Last modified: Sun  12/10/95
*
*        Purpose: To generate a user-specified number of
*                 random .DBF records for testing purposes.
*                 The program will generate random data for each field
*                 of a Clipper .DBF file, regardless of the data type.
*                 The input database must already exist.  Records will
*                 be APPENDED to the file.
*                 "It's not elegant, but it works!"
*
*         Syntax: RANDDAT <filename>  <# of records>
*
*          Notes: Compile with /n
*
*     This program is herewith placed in the public domain
*
**********************************************************************


FUNCTION main(cFile, cRecs)

LOCAL cChar         ,;
      Getlist  := {},;
      aStruct  := {},;     // Structure of target dbf
      nRecs    := 0 ,;     // Number of records to create
      nCounter := 0 ,;     // Loop/progress counter
      cType         ,;     // Field info
      cLen          ,;     //   "    "
      nDigits       ,;     //   "    "
      n             ,;     // Counter
      i                    //    "


? "-------------------------"
? "      Clipper .DBF"
? "  Random Data Generator"
? "-------------------------"
?

// Check parameters
IF EMPTY(cFile)
   ? 'Syntax is "RANDDAT <filename> <# of records>"'
   ?
   QUIT
ELSEIF AT(".", cFile) = 0
   cFile += ".DBF"
ELSEIF AT(".DBF", UPPER(cFile)) = 0
   ? "File must be a .DBF"
   ?
   QUIT
ENDIF
IF ! File(cFile)
   ? UPPER(cFile) + " does not exist"
   ?
   QUIT
ELSEIF EMPTY(cRecs) .OR. VAL(cRecs) = 0
   ? 'No record quantity specified'
   ? 'Syntax is "RANDDAT <filename> <# of records>"'
   ?
   QUIT
ENDIF


USE (cFile)
nRecs := VAL(cRecs)


// Stuff the structure to an array
aStruct := (cFile)->(DBSTRUCT())

? "Records generated:"



// Main loop
DO WHILE nCounter < nRecs

   (cFile)->(DBAPPEND())

   FOR n = 1 to FCOUNT()

     // Get field type and length
     cType   :=  aStruct[n,2]
     cLen    :=  aStruct[n,3]
     nDigits :=  aStruct[n,3] - aStruct[n,4] - 1


      DO CASE

      CASE cType = 'C'
         cChar := ''
         // Build string from random numbers. I could use INT(),
         // but it isn't required, and would only slow the loop.
         FOR i = 1 TO cLen
            cChar += CHR(ft_rand1(26)+65)
         NEXT
         FIELDPUT(n, cChar)

      CASE cType = 'N'
         FIELDPUT(n, ft_rand1(INT(VAL(LEFT('999999999',nDigits))  )))

      CASE cType = 'D'
         FIELDPUT(n, DATE() - ft_rand1(9999))

      CASE cType = 'L'
         FIELDPUT(n, SECONDS() - INT(SECONDS()) >.49)

      CASE cType = 'M'
         FIELDPUT(n, "Seconds today: " + STR(SECONDS()))

      ENDCASE


   NEXT

   @ ROW(), 17 SAY ++nCounter
ENDDO

?
QUIT
RETURN nil




// This random number generator is from the Nanforum LIB v3.05
// Thanks to the author for contributing it to the public domain


 * File......: RAND1.PRG
 * Author....: Gary Baren
 * CIS ID....: 75470,1027
 *
 * This is an original work by Gary Baren and is hereby placed in the
 * public domain.


function ft_rand1(nMax)
  STATIC nSeed
  LOCAL m := 100000000, b := 31415621

  nSeed := IIF( nSeed == NIL, SECONDS(), nSeed )   // init_seed()

  RETURN( nMax * ( ( nSeed := MOD( nSeed*b+1, m ) ) / m ) )


